# ------------------------------------------------------------
# Copyright (c) Microsoft Corporation.  All rights reserved.
# Licensed under the MIT License (MIT). See License.txt in the repo root for license information.
# ------------------------------------------------------------

param (
    [Parameter(Mandatory=$false)]
    [switch] $KeepFabricData,
    
    [Parameter(Mandatory=$false)]
    [string] $LogPath,

    [Parameter(Mandatory=$false)]
    [string] $FabricDataRoot
)

function DeletePath([string] $pathToDelete)
{
    if (([string]::IsNullOrEmpty($pathToDelete)) -or (!(Test-Path $pathToDelete)))
    {
        return
    }

    Write-Host "Deleting $pathToDelete"
    Remove-Item $pathToDelete -Recurse -Force 2> $null
    if (Test-Path $pathToDelete)
    {
        Write-Warning "$pathToDelete still exists. Delete manually."
    }
}

$Identity = [System.Security.Principal.WindowsIdentity]::GetCurrent()
$Principal = New-Object System.Security.Principal.WindowsPrincipal($Identity)
$IsAdmin = $Principal.IsInRole([System.Security.Principal.WindowsBuiltInRole]::Administrator)

if (!$IsAdmin)
{
    Write-Host "Please run the script with administrative privileges." -ForegroundColor "Red"
    exit 1
}

# Get registry keys
$FabricRegKeyPath = "HKLM:\SOFTWARE\Microsoft\Service Fabric"
Get-ItemProperty -Path $FabricRegKeyPath -ErrorAction Ignore | Out-Null
if ($?)
{
    $FabricRoot = (Get-ItemProperty -Path $FabricRegKeyPath -Name "FabricRoot").FabricRoot
    $FabricCodePath = (Get-ItemProperty -Path $FabricRegKeyPath -Name "FabricCodePath").FabricCodePath
    $FabricDataRoot = (Get-ItemProperty -Path $FabricRegKeyPath -Name "FabricDataRoot").FabricDataRoot
    $FabricBinRoot = (Get-ItemProperty -Path $FabricRegKeyPath -Name "FabricBinRoot").FabricBinRoot
}
else
{
    Write-Host "Registry key is already removed."
}

$gacDirectories = @(
    "$env:SystemRoot\Microsoft.NET\assembly\GAC_64\System.Fabric",
    "$env:SystemRoot\Microsoft.NET\assembly\GAC_64\System.Fabric.Strings",
    "$env:SystemRoot\Microsoft.NET\assembly\GAC_MSIL\System.Fabric.Management.ServiceModel",
    "$env:SystemRoot\Microsoft.NET\assembly\GAC_MSIL\System.Fabric.Strings" )

# Stop services
Get-Service FabricHostSvc -ErrorAction Ignore | Out-Null
if ($?)
{
    Write-Host "Stopping FabricHostSvc."
    Stop-Service FabricHostSvc
}
else
{
    Write-Host "FabricHostSvc is already removed."
}
Get-Service FabricInstallerSvc -ErrorAction Ignore | Out-Null
if ($?)
{
    Write-Host "Stopping FabricInstallerSvc."
    Stop-Service FabricInstallerSvc
}
else
{
    $FabricInstallerSvcAbsent = $true
    Write-Host "FabricInstallerSvc is already removed."
}

# Try determine file locations if registry key deleted
if (!(Test-Path variable:FabricRoot) `
    -or [string]::IsNullOrEmpty($FabricRoot) `
    -or !(Test-Path $FabricRoot))
{
    $AttemptedFabricRootPath = Join-Path $env:ProgramFiles "Microsoft Service Fabric"
    if (Test-Path $AttemptedFabricRootPath)
    {
        Write-Host "FabricRoot autodetermined at '$AttemptedFabricRootPath'." 
        $FabricRoot = $AttemptedFabricRootPath
        $AttemptedFabricCodePath = Join-Path $FabricRoot "bin\Fabric\Fabric.Code"
        if (Test-Path $AttemptedFabricCodePath)
        {
            Write-Host "FabricCodePath autodetermined at '$AttemptedFabricCodePath'." 
            $FabricCodePath = $AttemptedFabricCodePath
        }
        else
        {
            Write-Host "FabricCodePath not present."
        }
    }
    else
    {
        Write-Host "FabricRoot not present."
    }
}

if ([string]::IsNullOrEmpty($LogPath))
{
    $LogPath = Join-Path $env:TEMP -ChildPath "FabricSetupLog.Uninstall.log"
}

# Uninstall Fabric
if ((Test-Path variable:FabricCodePath) `
    -and ![string]::IsNullOrEmpty($FabricCodePath))
{
    $fabricSetupPath = Join-Path $FabricCodePath -ChildPath "FabricSetup.exe"
    if (Test-Path $fabricSetupPath)
    {
       $arguments = "/operation:uninstall"
       if ($KeepFabricData.IsPresent)
       {
            $arguments += " /deleteData:false"; 
       }

       Write-Host "FabricSetup uninstall running..."
       $command = "`"$fabricSetupPath`" $arguments && exit 0 || exit 1"
       $setupOutput = cmd.exe /c $command > "$LogPath"
       if ($LASTEXITCODE -eq 1)
       {
            Write-Error "FabricSetup ran into an issue. Find log at $LogPath."
            if ($setupOutput -is [System.Array])
            {
                $setupOutput = $setupOutput | Out-String
            }
            Write-Error $setupOutput
        }
        else
        {
             Write-Host "FabricSetup uninstall Completed. Find log at $LogPath."
        }
    }
    else
    {
        Write-Host "FabricSetup.exe is no longer in FabricCodePath. Uninstall may have already run."
    }
}
else
{
    Write-Host "Fabric is already uninstalled."
}

if (!(Test-Path variable:FabricInstallerSvcAbsent) `
    -or !$FabricInstallerSvcAbsent)
{
    Get-Service FabricInstallerSvc -ErrorAction Ignore | Out-Null
    if ($?)
    {
        Write-Host "Deleting FabricInstallerSvc"
        $service = Get-WmiObject -Class Win32_Service -Filter "Name='FabricInstallerSvc'"
        $service.delete() | Out-Null

        Get-Service FabricInstallerSvc -ErrorAction Ignore | Out-Null
        if ($?)
        {
            Write-Warning "FabricInstallerSvc is still present. May need to reboot to complete uninstallation."
        }
    }
    else
    {
        Write-Host "FabricInstallerSvc is already removed."
    }
}

# Delete cluster data & bits
if ((Test-Path variable:FabricDataRoot) `
    -and (!$KeepFabricData.IsPresent))
{
    DeletePath($FabricDataRoot)
}

if (Test-Path variable:FabricBinRoot) { DeletePath($FabricBinRoot) }
if (Test-Path variable:FabricRoot) { DeletePath($FabricRoot) }

foreach ($gacDirectory in $gacDirectories)
{
    DeletePath($gacDirectory)
}


# SIG # Begin signature block
# MIIjhwYJKoZIhvcNAQcCoIIjeDCCI3QCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCDGxr3nS3N8GQF+
# XH1ApgppgVuwoWyOxB+AQBJ52EpooaCCDXYwggX0MIID3KADAgECAhMzAAAB3vl+
# gOdHKPWkAAAAAAHeMA0GCSqGSIb3DQEBCwUAMH4xCzAJBgNVBAYTAlVTMRMwEQYD
# VQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNy
# b3NvZnQgQ29ycG9yYXRpb24xKDAmBgNVBAMTH01pY3Jvc29mdCBDb2RlIFNpZ25p
# bmcgUENBIDIwMTEwHhcNMjAxMjE1MjEzMTQ0WhcNMjExMjAyMjEzMTQ0WjB0MQsw
# CQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9u
# ZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMR4wHAYDVQQDExVNaWNy
# b3NvZnQgQ29ycG9yYXRpb24wggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIB
# AQC42o7GuqPBrC9Z9N+JtpXANgk2m77zmZSuuBKQmr5pZRmQCht/u/V21N5nwBWK
# NGwCZNdI98dyYGYORRZgrMOh8JWxDBjLMQYtqklGLw5ZPw3OCGCIM2ZU0snDlvZ3
# nKwys5NtPlY4shJxcVM2dhMnXhRTqvtexmeWpfmvtiop7jJn2Sdq0iDybDyU2vMz
# nH2ASetgjvuW2eP4d6zQXlboTBBu1ZxTv/aCRrWCWUPge8lHr3wtiPJHMyxmRHXT
# ulS2VksZ6iI9RLOdlqup9UOcnKRaj1usJKjwADu75+fegAZ4HPWSEXXmpBmuhvbT
# Euwa04eiL7ZKbG3mY9EqpiJ7AgMBAAGjggFzMIIBbzAfBgNVHSUEGDAWBgorBgEE
# AYI3TAgBBggrBgEFBQcDAzAdBgNVHQ4EFgQUbrkwVx/G26M/PsNzHEotPDOdBMcw
# RQYDVR0RBD4wPKQ6MDgxHjAcBgNVBAsTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEW
# MBQGA1UEBRMNMjMwMDEyKzQ2MzAwODAfBgNVHSMEGDAWgBRIbmTlUAXTgqoXNzci
# tW2oynUClTBUBgNVHR8ETTBLMEmgR6BFhkNodHRwOi8vd3d3Lm1pY3Jvc29mdC5j
# b20vcGtpb3BzL2NybC9NaWNDb2RTaWdQQ0EyMDExXzIwMTEtMDctMDguY3JsMGEG
# CCsGAQUFBwEBBFUwUzBRBggrBgEFBQcwAoZFaHR0cDovL3d3dy5taWNyb3NvZnQu
# Y29tL3BraW9wcy9jZXJ0cy9NaWNDb2RTaWdQQ0EyMDExXzIwMTEtMDctMDguY3J0
# MAwGA1UdEwEB/wQCMAAwDQYJKoZIhvcNAQELBQADggIBAHBTJKafCqTZswwxIpvl
# yU+K/+9oxjswaMqV+yGkRLa7LDqf917yb+IHjsPphMwe0ncDkpnNtKazW2doVHh3
# wMNXUYX6DzyVg1Xr/MTYaai0/GkPR/RN4MSBfoVBDzXJSisnYEWlK1TbI1J1mNTU
# iyiaktveVsH3xQyOVXQEpKFW17xYoHGjYm8s5v22mRE/ShVgsEW9ckxeQbJPCkPc
# PiqD4eXwPguTxv06Pwxva8lsjsPDvo2EgwozBCNGRAxsv2pEl0bh+yOtaFpfQWG7
# yMskiLQwWWoWFyuzm6yiKmZ/jdfO98xR1bFUhQMdwQoMi0lCUMx6YQJj1WpNUTDq
# X0ttJGny2aPWsoOgZ5fzKHNfCowOA+7hLc6gCVRBzyMN/xvV19aKymPt8I/J5gqA
# ZCQT19YgNKyhHUYS4GnFyMr/0GCezE8kexDGeQ3JX1TpHQvcz/dghK30fWM9z44l
# BjNcMV/HtTuefSFsr9tCp53wVaw65LudxSjH+/a2zUa85KKCBzj/GU4OhDaa5Wd4
# 8jr0JSm/515Ynzm1Xje5Ai/qo9xaGCrjrVcJUxBXd/SZPorm3HN6U1aJnL2Kw6nY
# 8Rs205CIWT28aFTecMQ6+KnMt1NZR4pogBnnpWSLc92JMbUd1Z6IbauU6U/oOjyl
# WOtkYUKbyE7EvK9GwUQXMds/MIIHejCCBWKgAwIBAgIKYQ6Q0gAAAAAAAzANBgkq
# hkiG9w0BAQsFADCBiDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24x
# EDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
# bjEyMDAGA1UEAxMpTWljcm9zb2Z0IFJvb3QgQ2VydGlmaWNhdGUgQXV0aG9yaXR5
# IDIwMTEwHhcNMTEwNzA4MjA1OTA5WhcNMjYwNzA4MjEwOTA5WjB+MQswCQYDVQQG
# EwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwG
# A1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgwJgYDVQQDEx9NaWNyb3NvZnQg
# Q29kZSBTaWduaW5nIFBDQSAyMDExMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIIC
# CgKCAgEAq/D6chAcLq3YbqqCEE00uvK2WCGfQhsqa+laUKq4BjgaBEm6f8MMHt03
# a8YS2AvwOMKZBrDIOdUBFDFC04kNeWSHfpRgJGyvnkmc6Whe0t+bU7IKLMOv2akr
# rnoJr9eWWcpgGgXpZnboMlImEi/nqwhQz7NEt13YxC4Ddato88tt8zpcoRb0Rrrg
# OGSsbmQ1eKagYw8t00CT+OPeBw3VXHmlSSnnDb6gE3e+lD3v++MrWhAfTVYoonpy
# 4BI6t0le2O3tQ5GD2Xuye4Yb2T6xjF3oiU+EGvKhL1nkkDstrjNYxbc+/jLTswM9
# sbKvkjh+0p2ALPVOVpEhNSXDOW5kf1O6nA+tGSOEy/S6A4aN91/w0FK/jJSHvMAh
# dCVfGCi2zCcoOCWYOUo2z3yxkq4cI6epZuxhH2rhKEmdX4jiJV3TIUs+UsS1Vz8k
# A/DRelsv1SPjcF0PUUZ3s/gA4bysAoJf28AVs70b1FVL5zmhD+kjSbwYuER8ReTB
# w3J64HLnJN+/RpnF78IcV9uDjexNSTCnq47f7Fufr/zdsGbiwZeBe+3W7UvnSSmn
# Eyimp31ngOaKYnhfsi+E11ecXL93KCjx7W3DKI8sj0A3T8HhhUSJxAlMxdSlQy90
# lfdu+HggWCwTXWCVmj5PM4TasIgX3p5O9JawvEagbJjS4NaIjAsCAwEAAaOCAe0w
# ggHpMBAGCSsGAQQBgjcVAQQDAgEAMB0GA1UdDgQWBBRIbmTlUAXTgqoXNzcitW2o
# ynUClTAZBgkrBgEEAYI3FAIEDB4KAFMAdQBiAEMAQTALBgNVHQ8EBAMCAYYwDwYD
# VR0TAQH/BAUwAwEB/zAfBgNVHSMEGDAWgBRyLToCMZBDuRQFTuHqp8cx0SOJNDBa
# BgNVHR8EUzBRME+gTaBLhklodHRwOi8vY3JsLm1pY3Jvc29mdC5jb20vcGtpL2Ny
# bC9wcm9kdWN0cy9NaWNSb29DZXJBdXQyMDExXzIwMTFfMDNfMjIuY3JsMF4GCCsG
# AQUFBwEBBFIwUDBOBggrBgEFBQcwAoZCaHR0cDovL3d3dy5taWNyb3NvZnQuY29t
# L3BraS9jZXJ0cy9NaWNSb29DZXJBdXQyMDExXzIwMTFfMDNfMjIuY3J0MIGfBgNV
# HSAEgZcwgZQwgZEGCSsGAQQBgjcuAzCBgzA/BggrBgEFBQcCARYzaHR0cDovL3d3
# dy5taWNyb3NvZnQuY29tL3BraW9wcy9kb2NzL3ByaW1hcnljcHMuaHRtMEAGCCsG
# AQUFBwICMDQeMiAdAEwAZQBnAGEAbABfAHAAbwBsAGkAYwB5AF8AcwB0AGEAdABl
# AG0AZQBuAHQALiAdMA0GCSqGSIb3DQEBCwUAA4ICAQBn8oalmOBUeRou09h0ZyKb
# C5YR4WOSmUKWfdJ5DJDBZV8uLD74w3LRbYP+vj/oCso7v0epo/Np22O/IjWll11l
# hJB9i0ZQVdgMknzSGksc8zxCi1LQsP1r4z4HLimb5j0bpdS1HXeUOeLpZMlEPXh6
# I/MTfaaQdION9MsmAkYqwooQu6SpBQyb7Wj6aC6VoCo/KmtYSWMfCWluWpiW5IP0
# wI/zRive/DvQvTXvbiWu5a8n7dDd8w6vmSiXmE0OPQvyCInWH8MyGOLwxS3OW560
# STkKxgrCxq2u5bLZ2xWIUUVYODJxJxp/sfQn+N4sOiBpmLJZiWhub6e3dMNABQam
# ASooPoI/E01mC8CzTfXhj38cbxV9Rad25UAqZaPDXVJihsMdYzaXht/a8/jyFqGa
# J+HNpZfQ7l1jQeNbB5yHPgZ3BtEGsXUfFL5hYbXw3MYbBL7fQccOKO7eZS/sl/ah
# XJbYANahRr1Z85elCUtIEJmAH9AAKcWxm6U/RXceNcbSoqKfenoi+kiVH6v7RyOA
# 9Z74v2u3S5fi63V4GuzqN5l5GEv/1rMjaHXmr/r8i+sLgOppO6/8MO0ETI7f33Vt
# Y5E90Z1WTk+/gFcioXgRMiF670EKsT/7qMykXcGhiJtXcVZOSEXAQsmbdlsKgEhr
# /Xmfwb1tbWrJUnMTDXpQzTGCFWcwghVjAgEBMIGVMH4xCzAJBgNVBAYTAlVTMRMw
# EQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVN
# aWNyb3NvZnQgQ29ycG9yYXRpb24xKDAmBgNVBAMTH01pY3Jvc29mdCBDb2RlIFNp
# Z25pbmcgUENBIDIwMTECEzMAAAHe+X6A50co9aQAAAAAAd4wDQYJYIZIAWUDBAIB
# BQCgga4wGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEO
# MAwGCisGAQQBgjcCARUwLwYJKoZIhvcNAQkEMSIEIPafaxS0meyHyGv8CYqifvdv
# zTOFuUDbfJYMAs2IIO2QMEIGCisGAQQBgjcCAQwxNDAyoBSAEgBNAGkAYwByAG8A
# cwBvAGYAdKEagBhodHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20wDQYJKoZIhvcNAQEB
# BQAEggEAr2o6UiEDIuX77iE/vD6WT0PsF7dvyHqdRNlqnoYV4Qo4o7SKe6mi+Rzm
# oe8WSpiTXNU0riEdEWNjvefPXGpvoKK+FfVa3MW0cVoVQc2OWUH/Le6xurrQZ6JH
# ZiQR36DrJLwGyA19XG0y/i4Yhbf3svBR8EJedz1ucjSkJNWqqpewpGO0IX4j5iEn
# efJHxloffS+PdD1dRTerEbj61OSW0pZbD6siYrTQCAXXJgeHywS42noFPR32Q1xX
# yS0k0dN8f95IdcY3XrmFAPP323YtiJOX30GJLHTUtIMseXlFI4hAT+c1NnFHJ4kC
# MuubuheQFOHlpn8chagPtGK9ioavoqGCEvEwghLtBgorBgEEAYI3AwMBMYIS3TCC
# EtkGCSqGSIb3DQEHAqCCEsowghLGAgEDMQ8wDQYJYIZIAWUDBAIBBQAwggFVBgsq
# hkiG9w0BCRABBKCCAUQEggFAMIIBPAIBAQYKKwYBBAGEWQoDATAxMA0GCWCGSAFl
# AwQCAQUABCC14uhGC8jsIpJio3yYwSzOPWWLcu81NYjorUMBhi2guQIGYUTUZrie
# GBMyMDIxMTAwODIyMTIzMS4wNDJaMASAAgH0oIHUpIHRMIHOMQswCQYDVQQGEwJV
# UzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UE
# ChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSkwJwYDVQQLEyBNaWNyb3NvZnQgT3Bl
# cmF0aW9ucyBQdWVydG8gUmljbzEmMCQGA1UECxMdVGhhbGVzIFRTUyBFU046RjdB
# Ni1FMjUxLTE1MEExJTAjBgNVBAMTHE1pY3Jvc29mdCBUaW1lLVN0YW1wIFNlcnZp
# Y2Wggg5EMIIE9TCCA92gAwIBAgITMwAAAVmf/H5fLOryQwAAAAABWTANBgkqhkiG
# 9w0BAQsFADB8MQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4G
# A1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSYw
# JAYDVQQDEx1NaWNyb3NvZnQgVGltZS1TdGFtcCBQQ0EgMjAxMDAeFw0yMTAxMTQx
# OTAyMTVaFw0yMjA0MTExOTAyMTVaMIHOMQswCQYDVQQGEwJVUzETMBEGA1UECBMK
# V2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0
# IENvcnBvcmF0aW9uMSkwJwYDVQQLEyBNaWNyb3NvZnQgT3BlcmF0aW9ucyBQdWVy
# dG8gUmljbzEmMCQGA1UECxMdVGhhbGVzIFRTUyBFU046RjdBNi1FMjUxLTE1MEEx
# JTAjBgNVBAMTHE1pY3Jvc29mdCBUaW1lLVN0YW1wIFNlcnZpY2UwggEiMA0GCSqG
# SIb3DQEBAQUAA4IBDwAwggEKAoIBAQCueMRhyWfEh0RLTSKAaPQBujK6PIBGQrfF
# oFf5jXSLdrTDj981WFOPrEbCJsU8H8yRAwVmk3Oy8ZRMv9d9Nn0Znf0dqcJ2O6ck
# /dMr2QJkEC2eg/n2hgcMIYua63v7ZgSXWwFxWfKi9iQ3OLcQZ99DK9QvAxQXayI8
# Gz/otkXQDmksCLP8ULDHmQM97+Y/VRHcKvPojOmHC3Kiq2AMD/jhOfN+9Uk+ZI9n
# +6rk6Hk14Urw3MymK1aJC92Z9PijQJ26aeKx9bV8ppoF0HIFQs9RPxMvDRLL2dRY
# 1eUD+qLwzE/GAKOys2mL0+CMfsTFb1vtf9TJ2GmqEfGy50MZk2TjAgMBAAGjggEb
# MIIBFzAdBgNVHQ4EFgQU9tCphUa8rfrk6yfXiMI8suk3Y+cwHwYDVR0jBBgwFoAU
# 1WM6XIoxkPNDe3xGG8UzaFqFbVUwVgYDVR0fBE8wTTBLoEmgR4ZFaHR0cDovL2Ny
# bC5taWNyb3NvZnQuY29tL3BraS9jcmwvcHJvZHVjdHMvTWljVGltU3RhUENBXzIw
# MTAtMDctMDEuY3JsMFoGCCsGAQUFBwEBBE4wTDBKBggrBgEFBQcwAoY+aHR0cDov
# L3d3dy5taWNyb3NvZnQuY29tL3BraS9jZXJ0cy9NaWNUaW1TdGFQQ0FfMjAxMC0w
# Ny0wMS5jcnQwDAYDVR0TAQH/BAIwADATBgNVHSUEDDAKBggrBgEFBQcDCDANBgkq
# hkiG9w0BAQsFAAOCAQEAjZFUEugPgYa3xjggFqNynLlGuHrLac8p/mS5ZIdKSZgC
# aeBFA6y1rInmAn9qMqHPCo1TAvjRxRVbxy60jIVp0eRbWOpd2elK/SEwUs+uI+cE
# 0URPLyKUIh1WI0VTTxkdrYqhuZyj+frA9K2SOOuDhTc+J+3qwyTqVeyJtS/7AMH1
# /hh6HOI+a37gLkExWPNrHWL7RzOC08cFffe7oZRbOdqB2qXRVtSl7erzMRF50l/L
# KEH1HfjPmKtye7nXOkfeNRrsX3egNL3nFeeiY75qQp4OI0ZKrgHsn/3SpkFGkdXy
# rwCwUQJmZAFLVoo0v9zJHkL/5VLx1aOxjxcyfzt8CTCCBnEwggRZoAMCAQICCmEJ
# gSoAAAAAAAIwDQYJKoZIhvcNAQELBQAwgYgxCzAJBgNVBAYTAlVTMRMwEQYDVQQI
# EwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3Nv
# ZnQgQ29ycG9yYXRpb24xMjAwBgNVBAMTKU1pY3Jvc29mdCBSb290IENlcnRpZmlj
# YXRlIEF1dGhvcml0eSAyMDEwMB4XDTEwMDcwMTIxMzY1NVoXDTI1MDcwMTIxNDY1
# NVowfDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcT
# B1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEmMCQGA1UE
# AxMdTWljcm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIwMTAwggEiMA0GCSqGSIb3DQEB
# AQUAA4IBDwAwggEKAoIBAQCpHQ28dxGKOiDs/BOX9fp/aZRrdFQQ1aUKAIKF++18
# aEssX8XD5WHCdrc+Zitb8BVTJwQxH0EbGpUdzgkTjnxhMFmxMEQP8WCIhFRDDNdN
# uDgIs0Ldk6zWczBXJoKjRQ3Q6vVHgc2/JGAyWGBG8lhHhjKEHnRhZ5FfgVSxz5NM
# ksHEpl3RYRNuKMYa+YaAu99h/EbBJx0kZxJyGiGKr0tkiVBisV39dx898Fd1rL2K
# Qk1AUdEPnAY+Z3/1ZsADlkR+79BL/W7lmsqxqPJ6Kgox8NpOBpG2iAg16HgcsOmZ
# zTznL0S6p/TcZL2kAcEgCZN4zfy8wMlEXV4WnAEFTyJNAgMBAAGjggHmMIIB4jAQ
# BgkrBgEEAYI3FQEEAwIBADAdBgNVHQ4EFgQU1WM6XIoxkPNDe3xGG8UzaFqFbVUw
# GQYJKwYBBAGCNxQCBAweCgBTAHUAYgBDAEEwCwYDVR0PBAQDAgGGMA8GA1UdEwEB
# /wQFMAMBAf8wHwYDVR0jBBgwFoAU1fZWy4/oolxiaNE9lJBb186aGMQwVgYDVR0f
# BE8wTTBLoEmgR4ZFaHR0cDovL2NybC5taWNyb3NvZnQuY29tL3BraS9jcmwvcHJv
# ZHVjdHMvTWljUm9vQ2VyQXV0XzIwMTAtMDYtMjMuY3JsMFoGCCsGAQUFBwEBBE4w
# TDBKBggrBgEFBQcwAoY+aHR0cDovL3d3dy5taWNyb3NvZnQuY29tL3BraS9jZXJ0
# cy9NaWNSb29DZXJBdXRfMjAxMC0wNi0yMy5jcnQwgaAGA1UdIAEB/wSBlTCBkjCB
# jwYJKwYBBAGCNy4DMIGBMD0GCCsGAQUFBwIBFjFodHRwOi8vd3d3Lm1pY3Jvc29m
# dC5jb20vUEtJL2RvY3MvQ1BTL2RlZmF1bHQuaHRtMEAGCCsGAQUFBwICMDQeMiAd
# AEwAZQBnAGEAbABfAFAAbwBsAGkAYwB5AF8AUwB0AGEAdABlAG0AZQBuAHQALiAd
# MA0GCSqGSIb3DQEBCwUAA4ICAQAH5ohRDeLG4Jg/gXEDPZ2joSFvs+umzPUxvs8F
# 4qn++ldtGTCzwsVmyWrf9efweL3HqJ4l4/m87WtUVwgrUYJEEvu5U4zM9GASinbM
# QEBBm9xcF/9c+V4XNZgkVkt070IQyK+/f8Z/8jd9Wj8c8pl5SpFSAK84Dxf1L3mB
# ZdmptWvkx872ynoAb0swRCQiPM/tA6WWj1kpvLb9BOFwnzJKJ/1Vry/+tuWOM7ti
# X5rbV0Dp8c6ZZpCM/2pif93FSguRJuI57BlKcWOdeyFtw5yjojz6f32WapB4pm3S
# 4Zz5Hfw42JT0xqUKloakvZ4argRCg7i1gJsiOCC1JeVk7Pf0v35jWSUPei45V3ai
# caoGig+JFrphpxHLmtgOR5qAxdDNp9DvfYPw4TtxCd9ddJgiCGHasFAeb73x4QDf
# 5zEHpJM692VHeOj4qEir995yfmFrb3epgcunCaw5u+zGy9iCtHLNHfS4hQEegPsb
# iSpUObJb2sgNVZl6h3M7COaYLeqN4DMuEin1wC9UJyH3yKxO2ii4sanblrKnQqLJ
# zxlBTeCG+SqaoxFmMNO7dDJL32N79ZmKLxvHIa9Zta7cRDyXUHHXodLFVeNp3lfB
# 0d4wwP3M5k37Db9dT+mdHhk4L7zPWAUu7w2gUDXa7wknHNWzfjUeCLraNtvTX4/e
# dIhJEqGCAtIwggI7AgEBMIH8oYHUpIHRMIHOMQswCQYDVQQGEwJVUzETMBEGA1UE
# CBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9z
# b2Z0IENvcnBvcmF0aW9uMSkwJwYDVQQLEyBNaWNyb3NvZnQgT3BlcmF0aW9ucyBQ
# dWVydG8gUmljbzEmMCQGA1UECxMdVGhhbGVzIFRTUyBFU046RjdBNi1FMjUxLTE1
# MEExJTAjBgNVBAMTHE1pY3Jvc29mdCBUaW1lLVN0YW1wIFNlcnZpY2WiIwoBATAH
# BgUrDgMCGgMVACp2ywCPH4TufEglq6WZ171xGbIRoIGDMIGApH4wfDELMAkGA1UE
# BhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAc
# BgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEmMCQGA1UEAxMdTWljcm9zb2Z0
# IFRpbWUtU3RhbXAgUENBIDIwMTAwDQYJKoZIhvcNAQEFBQACBQDlCwGyMCIYDzIw
# MjExMDA4MjE0MzE0WhgPMjAyMTEwMDkyMTQzMTRaMHcwPQYKKwYBBAGEWQoEATEv
# MC0wCgIFAOULAbICAQAwCgIBAAICJU0CAf8wBwIBAAICEVQwCgIFAOUMUzICAQAw
# NgYKKwYBBAGEWQoEAjEoMCYwDAYKKwYBBAGEWQoDAqAKMAgCAQACAwehIKEKMAgC
# AQACAwGGoDANBgkqhkiG9w0BAQUFAAOBgQCdfZtBfkCiFWUHHsQLWENpwb3THQD1
# lYKdRwpVhkkcSwqLh20xMw4TS26xUzYig3jB+8CMlydEvLekQS0HizGqPerDOxQM
# zGEkCOX9wa3KAJBehsyEc7BXIgNEvaDHvx6xDEAxsmDH9zgSRqs9oCSGj2l8yB8Z
# OGU7Zh6c5n1sGTGCAw0wggMJAgEBMIGTMHwxCzAJBgNVBAYTAlVTMRMwEQYDVQQI
# EwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3Nv
# ZnQgQ29ycG9yYXRpb24xJjAkBgNVBAMTHU1pY3Jvc29mdCBUaW1lLVN0YW1wIFBD
# QSAyMDEwAhMzAAABWZ/8fl8s6vJDAAAAAAFZMA0GCWCGSAFlAwQCAQUAoIIBSjAa
# BgkqhkiG9w0BCQMxDQYLKoZIhvcNAQkQAQQwLwYJKoZIhvcNAQkEMSIEIBOwVbrc
# V6cRD4DgJVhZ2L9mcJmPsI0G3Nq9fgMmoYMfMIH6BgsqhkiG9w0BCRACLzGB6jCB
# 5zCB5DCBvQQgAVgbz78w5nzXks5iU+PnRgPro/6qu/3ypFJ9+q8TQ+gwgZgwgYCk
# fjB8MQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMH
# UmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSYwJAYDVQQD
# Ex1NaWNyb3NvZnQgVGltZS1TdGFtcCBQQ0EgMjAxMAITMwAAAVmf/H5fLOryQwAA
# AAABWTAiBCBuN2mRfR3PQTGJT+uVMDBDHESBRa4F1/3I3mfmreyfJjANBgkqhkiG
# 9w0BAQsFAASCAQB2AvJ1zC54M+q+EueVcBFjEGcJ4w4oq/OXUrVtIVL7P/MQS4VO
# 7b2w+xDEu7CfW4kDnmXSiFj9ns29JhDPNovotbjT4qfOfvdAd3Y1l2zebGoVHH8B
# jhWNWtlzpNAGyGb8XKX0/3uCy0TDm0TlYIzchIZDwbR4JDqfFRnMYRy0ZEYfYGp2
# sizGfrVU5EWEvw9+PkKsFGhxMNf+72v8bNQmEec82x24w8JeW0iTgH+QmYiueaDH
# YvGV7Q5nIAPj6Bp9jU5+qG520epa6ziPQM9shFR1dZejmWafQTcLQXBNmI4iofBK
# nP2QIHNY9dfqlK2kup/pAjSGfvYUXnEABHaz
# SIG # End signature block
